@@ -39,6 +39,7 @@
 #include <sys/types.h>
 #include <stdarg.h>
 #include <getopt.h>
+#include <unistd.h>
 
 #ifndef IGT_LOG_DOMAIN
 #define IGT_LOG_DOMAIN (NULL)
@@ -231,6 +232,7 @@ void __igt_subtest_group_restore(int);
 #define igt_main \
 	static void igt_tokencat(__real_main, __LINE__)(void); \
 	int main(int argc, char **argv) { \
+		sync(); \
 		igt_subtest_init_parse_opts(&argc, argv, NULL, NULL, NULL, \
 					    NULL, NULL); \
 		igt_tokencat(__real_main, __LINE__)(); \
@@ -271,6 +273,7 @@ void igt_simple_init_parse_opts(int *argc, char **argv,
 #define igt_simple_main \
 	static void igt_tokencat(__real_main, __LINE__)(void); \
 	int main(int argc, char **argv) { \
+		sync(); \
 		igt_simple_init_parse_opts(&argc, argv, NULL, NULL, NULL, \
 					   NULL, NULL); \
 		igt_tokencat(__real_main, __LINE__)(); \
@@ -278,6 +281,19 @@ void igt_simple_init_parse_opts(int *argc, char **argv,
 	} \
 	static void igt_tokencat(__real_main, __LINE__)(void) \
 
+/**
+ * igt_constructor:
+ *
+ * Convenience macro to run the provided code block when igt first starts,
+ * before any tests have been run. This should be used for parts of the igt
+ * library that require initialization of objects with global context.
+ *
+ * This code block will be executed exactly once.
+ */
+#define igt_constructor \
+	__attribute__((constructor)) \
+	static void igt_tokencat(__igt_constructor_l, __LINE__)(void)
+
 __attribute__((format(printf, 1, 2)))
 void igt_skip(const char *f, ...) __attribute__((noreturn));
 __attribute__((format(printf, 5, 6)))
@@ -402,6 +418,24 @@ static inline void igt_ignore_warn(bool value)
 				  "error: %#x " #ncmp " %#x\n", __n1, __n2); \
 	} while (0)
 
+/**
+ * igt_assert_cmps64:
+ * @n1: first value
+ * @cmp: compare operator
+ * @ncmp: negated version of @cmp
+ * @n2: second value
+ *
+ * Like igt_assert_cmpuint(), but for larger signed ints.
+ */
+#define igt_assert_cmps64(n1, cmp, ncmp, n2) \
+	do { \
+		int64_t __n1 = (n1), __n2 = (n2); \
+		if (__n1 cmp __n2) ; else \
+		__igt_fail_assert(IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, \
+				  #n1 " " #cmp " " #n2, \
+				  "error: %lld " #ncmp " %lld\n", (long long)__n1, (long long)__n2); \
+	} while (0)
+
 /**
  * igt_assert_cmpu64:
  * @n1: first value
@@ -460,6 +494,15 @@ static inline void igt_ignore_warn(bool value)
  */
 #define igt_assert_eq_u32(n1, n2) igt_assert_cmpuint(n1, ==, !=, n2)
 
+/**
+ * igt_assert_eq_s64:
+ * @n1: first integer
+ * @n2: second integer
+ *
+ * Like igt_assert_eq_u32(), but for int64_t.
+ */
+#define igt_assert_eq_s64(n1, n2) igt_assert_cmps64(n1, ==, !=, n2)
+
 /**
  * igt_assert_eq_u64:
  * @n1: first integer
@@ -785,12 +828,16 @@ extern enum igt_log_level igt_log_level;
  * Should be used everywhere where a test checks results to decide about
  * printing warnings. This is useful to streamline the test logic since it
  * allows for a more flat code control flow, similar to igt_assert()
+ *
+ * This macro also returns the value of @condition.
  */
-#define igt_warn_on(condition) do {\
-		if (condition) \
-			igt_warn("Warning on condition %s in fucntion %s, file %s:%i\n", \
+#define igt_warn_on(condition) ({ \
+		typeof(condition) ret__ = (condition); \
+		if (ret__) \
+			igt_warn("Warning on condition %s in function %s, file %s:%i\n", \
 				 #condition, __func__, __FILE__, __LINE__); \
-	} while (0)
+		ret__; \
+	})
 
 /**
  * igt_warn_on_f:
@@ -807,15 +854,18 @@ extern enum igt_log_level igt_log_level;
  *
  * In addition to the plain igt_warn_on_f() helper this allows to print
  * additional information (again as warnings) to help debugging test failures.
+ *
+ * It also returns the value of @condition.
  */
-#define igt_warn_on_f(condition, f...) do {\
-		if (condition) {\
-			igt_warn("Warning on condition %s in fucntion %s, file %s:%i\n", \
+#define igt_warn_on_f(condition, f...) ({ \
+		typeof(condition) ret__ = (condition); \
+		if (ret__) {\
+			igt_warn("Warning on condition %s in function %s, file %s:%i\n", \
 				 #condition, __func__, __FILE__, __LINE__); \
 			igt_warn(f); \
 		} \
-	} while (0)
-
+		ret__; \
+	})
 
 void igt_set_timeout(unsigned int seconds,
 		     const char *op);
