@@ -43,23 +43,49 @@ typedef struct _cairo cairo_t;
 
 #include <i915_drm.h>
 
-/* helpers to create nice-looking framebuffers */
-struct igt_fb {
+/**
+ * igt_fb_t:
+ * @fb_id: KMS ID of the framebuffer
+ * @fd: DRM device fd this framebuffer is created on
+ * @gem_handle: GEM handler of the underlying backing storage
+ * @is_dumb: Whether this framebuffer was allocated using the dumb buffer API
+ * @drm_format: DRM FOURCC code
+ * @width: width in pixels
+ * @height: height in pixels
+ * @stride: line stride in bytes
+ * @tiling: tiling mode as a DRM framebuffer modifier
+ * @size: size in bytes of the underlying backing storage
+ * @cairo_surface: optionally attached cairo drawing surface
+ * @domain: current domain for cache flushing tracking on i915.ko
+ *
+ * Tracking structure for KMS framebuffer objects.
+ */
+typedef struct igt_fb {
 	uint32_t fb_id;
+	int fd;
 	uint32_t gem_handle;
+	bool is_dumb;
 	uint32_t drm_format;
 	int width;
 	int height;
-	unsigned stride;
+	unsigned int stride;
 	uint64_t tiling;
-	unsigned size;
+	unsigned int size;
 	cairo_surface_t *cairo_surface;
-	uint32_t src_x;
-	uint32_t src_y;
-	uint32_t src_w;
-	uint32_t src_h;
-};
+	unsigned int domain;
+} igt_fb_t;
 
+/**
+ * igt_text_align:
+ * @align_left: align left
+ * @align_right: align right
+ * @align_bottom: align bottom
+ * @align_top: align top
+ * @align_vcenter: align vcenter
+ * @align_hcenter: align hcenter
+ *
+ * Alignment mode for text drawing using igt_cairo_printf_line().
+ */
 enum igt_text_align {
 	align_left,
 	align_bottom	= align_left,
@@ -96,6 +122,14 @@ unsigned int igt_create_image_fb(int drm_fd,  int width, int height,
 unsigned int igt_create_stereo_fb(int drm_fd, drmModeModeInfo *mode,
 				  uint32_t format, uint64_t tiling);
 void igt_remove_fb(int fd, struct igt_fb *fb);
+int igt_dirty_fb(int fd, struct igt_fb *fb);
+
+int igt_create_bo_with_dimensions(int fd, int width, int height, uint32_t format,
+				  uint64_t modifier, unsigned stride,
+				  unsigned *stride_ret, unsigned *size_ret,
+				  bool *is_dumb);
+
+uint64_t igt_fb_mod_to_tiling(uint64_t modifier);
 
 /* cairo-based painting */
 cairo_t *igt_get_cairo_ctx(int fd, struct igt_fb *fb);
@@ -105,6 +139,9 @@ void igt_paint_color_alpha(cairo_t *cr, int x, int y, int w, int h,
 			       double r, double g, double b, double a);
 void igt_paint_color_gradient(cairo_t *cr, int x, int y, int w, int h,
 				  int r, int g, int b);
+void igt_paint_color_gradient_range(cairo_t *cr, int x, int y, int w, int h,
+				    double sr, double sg, double sb,
+				    double er, double eg, double eb);
 void igt_paint_test_pattern(cairo_t *cr, int width, int height);
 void igt_paint_image(cairo_t *cr, const char *filename,
 			 int dst_x, int dst_y, int dst_width, int dst_height);
