@@ -38,6 +38,10 @@
 #elif HAVE_SYS_KD_H
 #include <sys/kd.h>
 #endif
+#ifdef HAVE_UDEV
+#include <libudev.h>
+#include <poll.h>
+#endif
 #include <errno.h>
 #include <time.h>
 
@@ -49,6 +53,7 @@
 #include "intel_chipset.h"
 #include "igt_debugfs.h"
 #include "igt_sysfs.h"
+#include "sw_sync.h"
 
 /**
  * SECTION:igt_kms
@@ -153,24 +158,7 @@ const unsigned char* igt_kms_get_base_edid(void)
 #define EDID_NAME alt_edid
 #include "igt_edid_template.h"
 
-/**
- * igt_kms_get_alt_edid:
- *
- * Get an alternate edid block, which includes the following modes:
- *
- *  - 1400x1050 60Hz
- *  - 1920x1080 60Hz
- *  - 1280x720 60Hz
- *  - 1024x768 60Hz
- *  - 800x600 60Hz
- *  - 640x480 60Hz
- *
- * This can be extended with further features using functions such as
- * #kmstest_edid_add_3d.
- *
- * Returns: an alternate edid block
- */
-static const char *igt_plane_prop_names[IGT_NUM_PLANE_PROPS] = {
+const char *igt_plane_prop_names[IGT_NUM_PLANE_PROPS] = {
 	"SRC_X",
 	"SRC_Y",
 	"SRC_W",
@@ -181,20 +169,22 @@ static const char *igt_plane_prop_names[IGT_NUM_PLANE_PROPS] = {
 	"CRTC_H",
 	"FB_ID",
 	"CRTC_ID",
+	"IN_FENCE_FD",
 	"type",
 	"rotation"
 };
 
-static const char *igt_crtc_prop_names[IGT_NUM_CRTC_PROPS] = {
+const char *igt_crtc_prop_names[IGT_NUM_CRTC_PROPS] = {
 	"background_color",
 	"CTM",
 	"DEGAMMA_LUT",
 	"GAMMA_LUT",
 	"MODE_ID",
-	"ACTIVE"
+	"ACTIVE",
+	"OUT_FENCE_PTR"
 };
 
-static const char *igt_connector_prop_names[IGT_NUM_CONNECTOR_PROPS] = {
+const char *igt_connector_prop_names[IGT_NUM_CONNECTOR_PROPS] = {
 	"scaling mode",
 	"CRTC_ID"
 };
@@ -297,6 +287,23 @@ igt_atomic_fill_pipe_props(igt_display_t *display, igt_pipe_t *pipe,
 	drmModeFreeObjectProperties(props);
 }
 
+/**
+ * igt_kms_get_alt_edid:
+ *
+ * Get an alternate edid block, which includes the following modes:
+ *
+ *  - 1400x1050 60Hz
+ *  - 1920x1080 60Hz
+ *  - 1280x720 60Hz
+ *  - 1024x768 60Hz
+ *  - 800x600 60Hz
+ *  - 640x480 60Hz
+ *
+ * This can be extended with further features using functions such as
+ * #kmstest_edid_add_3d.
+ *
+ * Returns: an alternate edid block
+ */
 const unsigned char* igt_kms_get_alt_edid(void)
 {
 	update_edid_csum(alt_edid);
@@ -308,7 +315,7 @@ const unsigned char* igt_kms_get_alt_edid(void)
  * kmstest_pipe_name:
  * @pipe: display pipe
  *
- * Returns: String represnting @pipe, e.g. "A".
+ * Returns: String representing @pipe, e.g. "A".
  */
 const char *kmstest_pipe_name(enum pipe pipe)
 {
@@ -324,29 +331,40 @@ const char *kmstest_pipe_name(enum pipe pipe)
 }
 
 /**
- * kmstest_plane_name:
+ * kmstest_pipe_to_index:
+ *@pipe: display pipe in string format
+ *
+ * Returns: index to corresponding pipe
+ */
+int kmstest_pipe_to_index(char pipe)
+{
+	if (pipe == 'A')
+		return 0;
+	else if (pipe == 'B')
+		return 1;
+	else if (pipe == 'C')
+		return 2;
+	else
+		return -EINVAL;
+}
+
+/**
+ * kmstest_plane_type_name:
  * @plane: display plane
  *
- * Returns: String represnting @pipe, e.g. "plane1".
+ * Returns: String representing @plane, e.g. "overlay".
  */
-const char *kmstest_plane_name(enum igt_plane plane)
+const char *kmstest_plane_type_name(int plane_type)
 {
 	static const char *names[] = {
-		[IGT_PLANE_1] = "plane1",
-		[IGT_PLANE_2] = "plane2",
-		[IGT_PLANE_3] = "plane3",
-		[IGT_PLANE_4] = "plane4",
-		[IGT_PLANE_5] = "plane5",
-		[IGT_PLANE_6] = "plane6",
-		[IGT_PLANE_7] = "plane7",
-		[IGT_PLANE_8] = "plane8",
-		[IGT_PLANE_9] = "plane9",
-		[IGT_PLANE_CURSOR] = "cursor",
+		[DRM_PLANE_TYPE_OVERLAY] = "overlay",
+		[DRM_PLANE_TYPE_PRIMARY] = "primary",
+		[DRM_PLANE_TYPE_CURSOR] = "cursor",
 	};
 
-	igt_assert(plane < ARRAY_SIZE(names) && names[plane]);
+	igt_assert(plane_type < ARRAY_SIZE(names) && names[plane_type]);
 
-	return names[plane];
+	return names[plane_type];
 }
 
 static const char *mode_stereo_name(const drmModeModeInfo *mode)
@@ -623,15 +641,20 @@ bool kmstest_force_connector(int drm_fd, drmModeConnector *connector,
 	uint32_t devid;
 	int len, dir, idx;
 
-	devid = intel_get_drm_devid(drm_fd);
+	if (is_i915_device(drm_fd)) {
+		devid = intel_get_drm_devid(drm_fd);
 
-	/* forcing hdmi or dp connectors on HSW and BDW doesn't currently work,
-	 * so fail early to allow the test to skip if required */
-	if ((connector->connector_type == DRM_MODE_CONNECTOR_HDMIA ||
-	     connector->connector_type == DRM_MODE_CONNECTOR_HDMIB ||
-	     connector->connector_type == DRM_MODE_CONNECTOR_DisplayPort)
-	    && (IS_HASWELL(devid) || IS_BROADWELL(devid)))
-		return false;
+		/*
+		 * forcing hdmi or dp connectors on HSW and BDW doesn't
+		 * currently work, so fail early to allow the test to skip if
+		 * required
+		 */
+		if ((connector->connector_type == DRM_MODE_CONNECTOR_HDMIA ||
+		     connector->connector_type == DRM_MODE_CONNECTOR_HDMIB ||
+		     connector->connector_type == DRM_MODE_CONNECTOR_DisplayPort)
+		    && (IS_HASWELL(devid) || IS_BROADWELL(devid)))
+			return false;
+	}
 
 	switch (state) {
 	case FORCE_CONNECTOR_ON:
@@ -1007,6 +1030,40 @@ void kmstest_set_connector_dpms(int fd, drmModeConnector *connector, int mode)
 					       dpms, mode) == 0);
 }
 
+/**
+ * kmstest_set_connector_broadcast_rgb:
+ * @fd: DRM fd
+ * @connector: libdrm connector
+ * @mode: Broadcast RGB mode
+ *
+ * This function sets the Broadcast RGB prop of @connector to @mode, if there
+ * is one.
+ *
+ * Returns: true if we found and set the Broadcast RGB prop, false otherwise
+ */
+bool kmstest_set_connector_broadcast_rgb(int fd, drmModeConnector *connector,
+					 enum kmstest_broadcast_rgb_mode mode)
+{
+	uint32_t prop_id;
+	int ret;
+
+	ret = kmstest_get_property(fd, connector->connector_id,
+				   DRM_MODE_OBJECT_CONNECTOR, "Broadcast RGB",
+				   &prop_id, NULL, NULL);
+	if (!ret) {
+		igt_debug("Broadcast RGB property not found on %d\n",
+			  connector->connector_id);
+		return false;
+	}
+
+	igt_debug("Setting Broadcast RGB mode on connector %d to %d\n",
+		  connector->connector_id, mode);
+	ret = drmModeConnectorSetProperty(fd, connector->connector_id, prop_id,
+					  mode);
+
+	return ret == 0;
+}
+
 /**
  * kmstest_get_property:
  * @drm_fd: drm file descriptor
@@ -1084,6 +1141,7 @@ void kmstest_edid_add_3d(const unsigned char *edid, size_t length,
 	*new_length = length + 128;
 
 	new_edid = calloc(*new_length, sizeof(char));
+	igt_assert_f(new_edid, "Failed to allocate %zu bytes for edid\n", sizeof(new_length));
 	memcpy(new_edid, edid, length);
 	*new_edid_ptr = new_edid;
 
@@ -1176,6 +1234,167 @@ int kmstest_get_crtc_idx(drmModeRes *res, uint32_t crtc_id)
 	igt_assert(false);
 }
 
+static inline uint32_t pipe_select(int pipe)
+{
+	if (pipe > 1)
+		return pipe << DRM_VBLANK_HIGH_CRTC_SHIFT;
+	else if (pipe > 0)
+		return DRM_VBLANK_SECONDARY;
+	else
+		return 0;
+}
+
+unsigned int kmstest_get_vblank(int fd, int pipe, unsigned int flags)
+{
+	union drm_wait_vblank vbl;
+
+	memset(&vbl, 0, sizeof(vbl));
+	vbl.request.type = DRM_VBLANK_RELATIVE | pipe_select(pipe) | flags;
+	if (drmIoctl(fd, DRM_IOCTL_WAIT_VBLANK, &vbl))
+		return 0;
+
+	return vbl.reply.sequence;
+}
+
+static void get_plane(char *str, int type, struct kmstest_plane *plane)
+{
+	int ret;
+	char buf[256];
+
+	plane->type = type;
+	ret = sscanf(str + 12, "%d%*c %*s %[^n]s",
+		     &plane->id,
+		     buf);
+	igt_assert_eq(ret, 2);
+
+	ret = sscanf(buf + 9, "%4d%*c%4d%*c", &plane->pos_x, &plane->pos_y);
+	igt_assert_eq(ret, 2);
+
+	ret = sscanf(buf + 30, "%4d%*c%4d%*c", &plane->width, &plane->height);
+	igt_assert_eq(ret, 2);
+}
+
+static int parse_planes(FILE *fid, struct kmstest_plane *planes)
+{
+	char tmp[256];
+	int n_planes;
+
+	n_planes = 0;
+	while (fgets(tmp, 256, fid) != NULL) {
+		if (strstr(tmp, "type=PRI") != NULL) {
+			if (planes) {
+				get_plane(tmp, DRM_PLANE_TYPE_PRIMARY, &planes[n_planes]);
+				planes[n_planes].index = n_planes;
+			}
+			n_planes++;
+		} else if (strstr(tmp, "type=OVL") != NULL) {
+			if (planes) {
+				get_plane(tmp, DRM_PLANE_TYPE_OVERLAY, &planes[n_planes]);
+				planes[n_planes].index = n_planes;
+			}
+			n_planes++;
+		} else if (strstr(tmp, "type=CUR") != NULL) {
+			if (planes) {
+				get_plane(tmp, DRM_PLANE_TYPE_CURSOR, &planes[n_planes]);
+				planes[n_planes].index = n_planes;
+			}
+			n_planes++;
+			break;
+		}
+	}
+
+	return n_planes;
+}
+
+static void parse_crtc(char *info, struct kmstest_crtc *crtc)
+{
+	char buf[256];
+	int ret;
+	char pipe;
+
+	ret = sscanf(info + 4, "%d%*c %*s %c%*c %*s %s%*c",
+		     &crtc->id, &pipe, buf);
+	igt_assert_eq(ret, 3);
+
+	crtc->pipe = kmstest_pipe_to_index(pipe);
+	igt_assert(crtc->pipe >= 0);
+
+	ret = sscanf(buf + 6, "%d%*c%d%*c",
+		     &crtc->width, &crtc->height);
+	igt_assert_eq(ret, 2);
+}
+
+void kmstest_get_crtc(enum pipe pipe, struct kmstest_crtc *crtc)
+{
+	char tmp[256];
+	FILE *fid;
+	const char *mode = "r";
+	int ncrtc;
+	int line;
+	long int n;
+
+	fid = igt_debugfs_fopen("i915_display_info", mode);
+
+	igt_skip_on(fid == NULL);
+
+	ncrtc = 0;
+	line = 0;
+	while (fgets(tmp, 256, fid) != NULL) {
+		if ((strstr(tmp, "CRTC") != NULL) && (line > 0)) {
+			if (strstr(tmp, "active=yes") != NULL) {
+				crtc->active = true;
+				parse_crtc(tmp, crtc);
+
+				n = ftell(fid);
+				crtc->n_planes = parse_planes(fid, NULL);
+				crtc->planes = calloc(crtc->n_planes, sizeof(*crtc->planes));
+				igt_assert_f(crtc->planes, "Failed to allocate memory for %d planes\n", crtc->n_planes);
+
+				fseek(fid, n, SEEK_SET);
+				parse_planes(fid, crtc->planes);
+
+				if (crtc->pipe != pipe) {
+					free(crtc->planes);
+				} else {
+					ncrtc++;
+					break;
+				}
+			}
+		}
+
+		line++;
+	}
+
+	fclose(fid);
+
+	igt_skip_on(ncrtc == 0);
+}
+
+void igt_assert_plane_visible(enum pipe pipe, bool visibility)
+{
+	struct kmstest_crtc crtc;
+	int i;
+	bool visible;
+
+	kmstest_get_crtc(pipe, &crtc);
+
+	visible = true;
+	for (i = 0; i < crtc.n_planes; i++) {
+		if (crtc.planes[i].type == DRM_PLANE_TYPE_PRIMARY)
+			continue;
+
+		if (crtc.planes[i].pos_x > crtc.width) {
+			visible = false;
+			break;
+		} else if (crtc.planes[i].pos_y > crtc.height) {
+			visible = false;
+			break;
+		}
+	}
+
+	igt_assert_eq(visible, visibility);
+}
+
 /*
  * A small modeset API
  */
@@ -1228,11 +1447,9 @@ static void igt_output_refresh(igt_output_t *output, bool final)
 
 	kmstest_free_connector_config(&output->config);
 
-	output->valid = _kmstest_connector_config(display->drm_fd,
-						  output->id,
-						  crtc_idx_mask,
-						  &output->config,
-						  output->valid < 0);
+	_kmstest_connector_config(display->drm_fd, output->id, crtc_idx_mask,
+				  &output->config, output->force_reprobe);
+	output->force_reprobe = false;
 
 	if (!output->name && output->config.connector) {
 		drmModeConnector *c = output->config.connector;
@@ -1245,16 +1462,17 @@ static void igt_output_refresh(igt_output_t *output, bool final)
 		igt_atomic_fill_connector_props(display, output,
 			IGT_NUM_CONNECTOR_PROPS, igt_connector_prop_names);
 
-	if (!output->valid)
-		return;
-
 	if (output->use_override_mode)
 		output->config.default_mode = output->override_mode;
 
+	if (output->config.pipe == PIPE_NONE)
+		return;
+
 	LOG(display, "%s: Selecting pipe %s\n", output->name,
 	    kmstest_pipe_name(output->config.pipe));
 
-	display->pipes_in_use |= 1 << output->config.pipe;
+	if (final)
+		display->pipes_in_use |= 1 << output->config.pipe;
 }
 
 static bool
@@ -1340,6 +1558,8 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 	 * hardware and cannot change of time (for now, at least).
 	 */
 	display->n_pipes = resources->count_crtcs;
+	display->pipes = calloc(sizeof(igt_pipe_t), display->n_pipes);
+	igt_assert_f(display->pipes, "Failed to allocate memory for %d pipes\n", display->n_pipes);
 
 	drmSetClientCap(drm_fd, DRM_CLIENT_CAP_UNIVERSAL_PLANES, 1);
 	is_atomic = drmSetClientCap(drm_fd, DRM_CLIENT_CAP_ATOMIC, 1);
@@ -1349,14 +1569,17 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 	for (i = 0; i < display->n_pipes; i++) {
 		igt_pipe_t *pipe = &display->pipes[i];
 		igt_plane_t *plane;
-		int p = IGT_PLANE_2;
+		int p = 1;
 		int j, type;
-		uint8_t n_planes = 0;
+		uint8_t last_plane = 0, n_planes = 0;
 		uint64_t prop_value;
 
 		pipe->crtc_id = resources->crtcs[i];
 		pipe->display = display;
 		pipe->pipe = i;
+		pipe->plane_cursor = -1;
+		pipe->plane_primary = -1;
+		pipe->planes = NULL;
 
 		get_crtc_property(display->drm_fd, pipe->crtc_id,
 				    "background_color",
@@ -1382,6 +1605,25 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 
 		igt_atomic_fill_pipe_props(display, pipe, IGT_NUM_CRTC_PROPS, igt_crtc_prop_names);
 
+		/* count number of valid planes */
+		for (j = 0; j < plane_resources->count_planes; j++) {
+			drmModePlane *drm_plane;
+
+			drm_plane = drmModeGetPlane(display->drm_fd,
+						    plane_resources->planes[j]);
+			igt_assert(drm_plane);
+
+			if (drm_plane->possible_crtcs & (1 << i))
+				n_planes++;
+
+			drmModeFreePlane(drm_plane);
+		}
+
+		igt_assert_lte(0, n_planes);
+		pipe->planes = calloc(sizeof(igt_plane_t), n_planes);
+		igt_assert_f(pipe->planes, "Failed to allocate memory for %d planes\n", n_planes);
+		last_plane = n_planes - 1;
+
 		/* add the planes that can be used with that pipe */
 		for (j = 0; j < plane_resources->count_planes; j++) {
 			drmModePlane *drm_plane;
@@ -1397,35 +1639,26 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 
 			type = get_drm_plane_type(display->drm_fd,
 						  plane_resources->planes[j]);
-			switch (type) {
-			case DRM_PLANE_TYPE_PRIMARY:
-				plane = &pipe->planes[IGT_PLANE_PRIMARY];
-				plane->is_primary = 1;
-				plane->index = IGT_PLANE_PRIMARY;
-				display->has_universal_planes = 1;
-				break;
-			case DRM_PLANE_TYPE_CURSOR:
-				/*
-				 * Cursor should be the highest index in our
-				 * internal list, but we don't know what that
-				 * is yet.  Just stick it in the last slot
-				 * for now and we'll move it later, if
-				 * necessary.
-				 */
-				plane = &pipe->planes[IGT_PLANE_CURSOR];
-				plane->is_cursor = 1;
-				plane->index = IGT_PLANE_CURSOR;
-				display->has_universal_planes = 1;
-				break;
-			default:
+
+			if (type == DRM_PLANE_TYPE_PRIMARY && pipe->plane_primary == -1) {
+				plane = &pipe->planes[0];
+				plane->index = 0;
+				pipe->plane_primary = 0;
+			} else if (type == DRM_PLANE_TYPE_CURSOR && pipe->plane_cursor == -1) {
+				plane = &pipe->planes[last_plane];
+				plane->index = last_plane;
+				pipe->plane_cursor = last_plane;
+				display->has_cursor_plane = true;
+			} else {
 				plane = &pipe->planes[p];
 				plane->index = p++;
-				break;
 			}
 
-			n_planes++;
+			igt_assert_f(plane->index < n_planes, "n_planes < plane->index failed\n");
+			plane->type = type;
 			plane->pipe = pipe;
 			plane->drm_plane = drm_plane;
+			plane->fence_fd = -1;
 
 			if (is_atomic == 0) {
 				display->is_atomic = 1;
@@ -1440,49 +1673,23 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 			plane->rotation = (igt_rotation_t)prop_value;
 		}
 
-		if (display->has_universal_planes) {
-			/*
-			 * If we have universal planes, we should have both
-			 * primary and cursor planes setup now.
-			 */
-			igt_assert(pipe->planes[IGT_PLANE_PRIMARY].drm_plane &&
-				   pipe->planes[IGT_PLANE_CURSOR].drm_plane);
+		/*
+		 * At the bare minimum, we should expect to have a primary
+		 * plane, and it must be in slot 0.
+		 */
+		igt_assert_eq(pipe->plane_primary, 0);
 
-			/*
-			 * Cursor was put in the last slot.  If we have 0 or
-			 * only 1 sprite, that's the wrong slot and we need to
-			 * move it down.
-			 */
-			if (p != IGT_PLANE_CURSOR) {
-				pipe->planes[p] = pipe->planes[IGT_PLANE_CURSOR];
-				pipe->planes[p].index = p;
-				memset(&pipe->planes[IGT_PLANE_CURSOR], 0,
-				       sizeof *plane);
-			}
-		} else {
-			/*
-			 * No universal plane support.  Add drm_plane-less
-			 * primary and cursor planes.
-			 */
-			plane = &pipe->planes[IGT_PLANE_PRIMARY];
-			plane->pipe = pipe;
-			plane->index = IGT_PLANE_PRIMARY;
-			plane->is_primary = true;
-
-			plane = &pipe->planes[p];
-			plane->pipe = pipe;
-			plane->index = p;
-			plane->is_cursor = true;
-		}
+		/*
+		 * There should be no gaps. If there is, something happened
+		 * which we can't handle (e.g. all planes are cursors).
+		 */
+		igt_assert_eq(p, last_plane);
 
 		pipe->n_planes = n_planes;
 
 		for_each_plane_on_pipe(display, i, plane)
 			plane->fb_changed = true;
 
-		/* make sure we don't overflow the plane array */
-		igt_assert_lte(pipe->n_planes, IGT_MAX_PLANES);
-
 		pipe->mode_changed = true;
 	}
 
@@ -1493,7 +1700,7 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 	 */
 	display->n_outputs = resources->count_connectors;
 	display->outputs = calloc(display->n_outputs, sizeof(igt_output_t));
-	igt_assert(display->outputs);
+	igt_assert_f(display->outputs, "Failed to allocate memory for %d outputs\n", display->n_outputs);
 
 	for (i = 0; i < display->n_outputs; i++) {
 		igt_output_t *output = &display->outputs[i];
@@ -1502,7 +1709,7 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 		 * We don't assign each output a pipe unless
 		 * a pipe is set with igt_output_set_pipe().
 		 */
-		output->valid = -1;
+		output->force_reprobe = true;
 		output->pending_crtc_idx_mask = 0;
 		output->id = resources->connectors[i];
 		output->display = display;
@@ -1523,6 +1730,32 @@ int igt_display_get_n_pipes(igt_display_t *display)
 	return display->n_pipes;
 }
 
+/**
+ * igt_output_from_connector:
+ * @display: a pointer to an #igt_display_t structure
+ * @connector: a pointer to a drmModeConnector
+ *
+ * Finds the output corresponding to the given connector
+ *
+ * Returns: A #igt_output_t structure configured to use the connector, or NULL
+ * if none was found
+ */
+igt_output_t *igt_output_from_connector(igt_display_t *display,
+					drmModeConnector *connector)
+{
+	igt_output_t *output, *found = NULL;
+
+	for_each_connected_output(display, output) {
+		if (output->config.connector->connector_id ==
+		    connector->connector_id) {
+			found = output;
+			break;
+		}
+	}
+
+	return found;
+}
+
 static void igt_pipe_fini(igt_pipe_t *pipe)
 {
 	int i;
@@ -1535,13 +1768,16 @@ static void igt_pipe_fini(igt_pipe_t *pipe)
 			plane->drm_plane = NULL;
 		}
 	}
+
+	free(pipe->planes);
+	pipe->planes = NULL;
 }
 
 static void igt_output_fini(igt_output_t *output)
 {
-	if (output->valid > 0)
-		kmstest_free_connector_config(&output->config);
+	kmstest_free_connector_config(&output->config);
 	free(output->name);
+	output->name = NULL;
 }
 
 /**
@@ -1562,6 +1798,8 @@ void igt_display_fini(igt_display_t *display)
 		igt_output_fini(&display->outputs[i]);
 	free(display->outputs);
 	display->outputs = NULL;
+	free(display->pipes);
+	display->pipes = NULL;
 }
 
 static void igt_display_refresh(igt_display_t *display)
@@ -1621,20 +1859,41 @@ static igt_pipe_t *igt_output_get_driving_pipe(igt_output_t *output)
 	return &display->pipes[pipe];
 }
 
-static igt_plane_t *igt_pipe_get_plane(igt_pipe_t *pipe, enum igt_plane plane)
+static igt_plane_t *igt_pipe_get_plane(igt_pipe_t *pipe, int plane_idx)
 {
-	int idx;
+	igt_assert_f(plane_idx >= 0 && plane_idx < pipe->n_planes,
+		"Valid pipe->planes plane_idx not found, plane_idx=%d n_planes=%d",
+		plane_idx, pipe->n_planes);
 
-	/* Cursor plane is always the highest index */
-	if (plane == IGT_PLANE_CURSOR)
-		idx = pipe->n_planes - 1;
-	else {
-		igt_assert_f(plane >= 0 && plane < (pipe->n_planes),
-			     "plane=%d\n", plane);
-		idx = plane;
+	return &pipe->planes[plane_idx];
+}
+
+
+igt_plane_t *igt_pipe_get_plane_type(igt_pipe_t *pipe, int plane_type)
+{
+	int i, plane_idx = -1;
+
+	switch(plane_type) {
+	case DRM_PLANE_TYPE_CURSOR:
+		plane_idx = pipe->plane_cursor;
+		break;
+	case DRM_PLANE_TYPE_PRIMARY:
+		plane_idx = pipe->plane_primary;
+		break;
+	case DRM_PLANE_TYPE_OVERLAY:
+		for(i = 0; i < pipe->n_planes; i++)
+			if (pipe->planes[i].type == DRM_PLANE_TYPE_OVERLAY)
+			    plane_idx = i;
+		break;
+	default:
+		break;
 	}
 
-	return &pipe->planes[idx];
+	igt_assert_f(plane_idx >= 0 && plane_idx < pipe->n_planes,
+		"Valid pipe->planes idx not found. plane_idx=%d plane_type=%d n_planes=%d\n",
+		plane_idx, plane_type, pipe->n_planes);
+
+	return &pipe->planes[plane_idx];
 }
 
 static igt_output_t *igt_pipe_get_output(igt_pipe_t *pipe)
@@ -1681,7 +1940,7 @@ static uint32_t igt_plane_get_fb_gem_handle(igt_plane_t *plane)
 #define CHECK_RETURN(r, fail) {	\
 	if (r && !fail)		\
 		return r;	\
-	igt_assert(r == 0);	\
+	igt_assert_eq(r, 0);	\
 }
 
 
@@ -1712,6 +1971,11 @@ igt_atomic_prepare_plane_commit(igt_plane_t *plane, igt_pipe_t *pipe,
 	    plane->index,
 	    fb_id);
 
+	if (plane->fence_fd >= 0) {
+		uint64_t fence_fd = (int64_t) plane->fence_fd;
+		igt_atomic_populate_plane_req(req, plane, IGT_PLANE_IN_FENCE_FD, fence_fd);
+	}
+
 	if (plane->fb_changed) {
 		igt_atomic_populate_plane_req(req, plane, IGT_PLANE_CRTC_ID, fb_id ? crtc_id : 0);
 		igt_atomic_populate_plane_req(req, plane, IGT_PLANE_FB_ID, fb_id);
@@ -1979,9 +2243,9 @@ static int igt_plane_commit(igt_plane_t *plane,
 			    enum igt_commit_style s,
 			    bool fail_on_error)
 {
-	if (plane->is_cursor && s == COMMIT_LEGACY) {
+	if (plane->type == DRM_PLANE_TYPE_CURSOR && s == COMMIT_LEGACY) {
 		return igt_cursor_commit_legacy(plane, pipe, fail_on_error);
-	} else if (plane->is_primary && s == COMMIT_LEGACY) {
+	} else if (plane->type == DRM_PLANE_TYPE_PRIMARY && s == COMMIT_LEGACY) {
 		return igt_primary_plane_commit_legacy(plane, pipe,
 						       fail_on_error);
 	} else {
@@ -2096,6 +2360,14 @@ static void igt_atomic_prepare_crtc_commit(igt_pipe_t *pipe_obj, drmModeAtomicRe
 		igt_atomic_populate_crtc_req(req, pipe_obj, IGT_CRTC_ACTIVE, !!output);
 	}
 
+	pipe_obj->out_fence_fd = -1;
+	if (pipe_obj->out_fence_requested)
+	{
+		pipe_obj->out_fence_requested = false;
+		igt_atomic_populate_crtc_req(req, pipe_obj, IGT_CRTC_OUT_FENCE_PTR,
+		    (uint64_t)(uintptr_t) &pipe_obj->out_fence_fd);
+	}
+
 	/*
 	 *	TODO: Add all crtc level properties here
 	 */
@@ -2172,6 +2444,28 @@ static int igt_atomic_commit(igt_display_t *display, uint32_t flags, void *user_
 	}
 
 	ret = drmModeAtomicCommit(display->drm_fd, req, flags, user_data);
+	if (!ret) {
+
+		for_each_pipe(display, pipe) {
+			igt_pipe_t *pipe_obj = &display->pipes[pipe];
+			igt_plane_t *plane;
+
+			/* reset fence_fd to prevent it from being set for the next commit */
+			for_each_plane_on_pipe(display, pipe, plane) {
+				igt_plane_set_fence_fd(plane, -1);
+			}
+
+			if (pipe_obj->out_fence_fd == -1)
+				continue;
+
+			igt_assert(pipe_obj->out_fence_fd >= 0);
+			ret = sync_fence_wait(pipe_obj->out_fence_fd, 1000);
+			igt_assert(ret == 0);
+			close(pipe_obj->out_fence_fd);
+			pipe_obj->out_fence_fd = -1;
+		}
+	}
+
 	drmModeAtomicFree(req);
 	return ret;
 
@@ -2198,7 +2492,9 @@ display_commit_changed(igt_display_t *display, enum igt_commit_style s)
 			plane->position_changed = false;
 			plane->size_changed = false;
 
-			if (s != COMMIT_LEGACY || !(plane->is_primary || plane->is_cursor))
+			if (s != COMMIT_LEGACY ||
+			    !(plane->type == DRM_PLANE_TYPE_PRIMARY ||
+			      plane->type == DRM_PLANE_TYPE_CURSOR))
 				plane->rotation_changed = false;
 		}
 	}
@@ -2244,7 +2540,7 @@ static int do_display_commit(igt_display_t *display,
 			igt_pipe_t *pipe_obj = &display->pipes[pipe];
 			igt_output_t *output = igt_pipe_get_output(pipe_obj);
 
-			if (output && output->valid > 0)
+			if (output)
 				valid_outs++;
 
 			ret = igt_pipe_commit(pipe_obj, s, fail_on_error);
@@ -2444,7 +2740,7 @@ void igt_output_set_pipe(igt_output_t *output, enum pipe pipe)
 	igt_display_t *display = output->display;
 	igt_pipe_t *old_pipe;
 
-	igt_assert(output->valid != -1);
+	igt_assert(output->name);
 
 	if (output->pending_crtc_idx_mask) {
 		old_pipe = igt_output_get_driving_pipe(output);
@@ -2463,8 +2759,7 @@ void igt_output_set_pipe(igt_output_t *output, enum pipe pipe)
 		display->pipes[pipe].mode_changed = true;
 	}
 
-	if (pipe != output->config.pipe)
-		output->config.pipe_changed = true;
+	output->config.pipe_changed = true;
 
 	igt_output_refresh(output, false);
 }
@@ -2478,14 +2773,24 @@ void igt_output_set_scaling_mode(igt_output_t *output, uint64_t scaling_mode)
 	igt_require(output->config.atomic_props_connector[IGT_CONNECTOR_SCALING_MODE]);
 }
 
-igt_plane_t *igt_output_get_plane(igt_output_t *output, enum igt_plane plane)
+igt_plane_t *igt_output_get_plane(igt_output_t *output, int plane_idx)
 {
 	igt_pipe_t *pipe;
 
 	pipe = igt_output_get_driving_pipe(output);
 	igt_assert(pipe);
 
-	return igt_pipe_get_plane(pipe, plane);
+	return igt_pipe_get_plane(pipe, plane_idx);
+}
+
+igt_plane_t *igt_output_get_plane_type(igt_output_t *output, int plane_type)
+{
+	igt_pipe_t *pipe;
+
+	pipe = igt_output_get_driving_pipe(output);
+	igt_assert(pipe);
+
+	return igt_pipe_get_plane_type(pipe, plane_type);
 }
 
 void igt_plane_set_fb(igt_plane_t *plane, struct igt_fb *fb)
@@ -2522,6 +2827,24 @@ void igt_plane_set_fb(igt_plane_t *plane, struct igt_fb *fb)
 	plane->size_changed = true;
 }
 
+/**
+ * igt_plane_set_fence_fd:
+ * @plane: plane
+ * @fence_fd: fence fd, disable fence_fd by setting it to -1
+ *
+ * This function sets a fence fd to enable a commit to wait for some event to
+ * occur before completing.
+ */
+void igt_plane_set_fence_fd(igt_plane_t *plane, int fence_fd)
+{
+	close(plane->fence_fd);
+
+	if (fcntl(fence_fd, F_GETFD) != -1)
+		plane->fence_fd = dup(fence_fd);
+	else
+		plane->fence_fd = -1;
+}
+
 void igt_plane_set_position(igt_plane_t *plane, int x, int y)
 {
 	igt_pipe_t *pipe = plane->pipe;
@@ -2641,6 +2964,18 @@ void igt_plane_set_rotation(igt_plane_t *plane, igt_rotation_t rotation)
 	plane->rotation_changed = true;
 }
 
+/**
+ * igt_pipe_request_out_fence:
+ * @pipe: pipe which out fence will be requested for
+ *
+ * Marks this pipe for requesting an out fence at the next atomic commit
+ * will contain the fd number of the out fence created by KMS.
+ */
+void igt_pipe_request_out_fence(igt_pipe_t *pipe)
+{
+	pipe->out_fence_requested = true;
+}
+
 void
 igt_pipe_set_degamma_lut(igt_pipe_t *pipe, void *ptr, size_t length)
 {
@@ -2725,6 +3060,10 @@ void igt_enable_connectors(void)
 
 		/* Do a probe. This may be the first action after booting */
 		c = drmModeGetConnector(drm_fd, res->connectors[i]);
+		if (!c) {
+			igt_warn("Could not read connector %u: %m\n", res->connectors[i]);
+			continue;
+		}
 
 		/* don't attempt to force connectors that are already connected
 		 */
@@ -2760,6 +3099,113 @@ void igt_reset_connectors(void)
 			      "detect");
 }
 
+#ifdef HAVE_UDEV
+
+/**
+ * igt_watch_hotplug:
+ *
+ * Begin monitoring udev for sysfs hotplug events.
+ *
+ * Returns: a udev monitor for detecting hotplugs on
+ */
+struct udev_monitor *igt_watch_hotplug(void)
+{
+	struct udev *udev;
+	struct udev_monitor *mon;
+	int ret, flags, fd;
+
+	udev = udev_new();
+	igt_assert(udev != NULL);
+
+	mon = udev_monitor_new_from_netlink(udev, "udev");
+	igt_assert(mon != NULL);
+
+	ret = udev_monitor_filter_add_match_subsystem_devtype(mon,
+							      "drm",
+							      "drm_minor");
+	igt_assert_eq(ret, 0);
+	ret = udev_monitor_filter_update(mon);
+	igt_assert_eq(ret, 0);
+	ret = udev_monitor_enable_receiving(mon);
+	igt_assert_eq(ret, 0);
+
+	/* Set the fd for udev as non blocking */
+	fd = udev_monitor_get_fd(mon);
+	flags = fcntl(fd, F_GETFL, 0);
+	igt_assert(flags);
+
+	flags |= O_NONBLOCK;
+	igt_assert_neq(fcntl(fd, F_SETFL, flags), -1);
+
+	return mon;
+}
+
+/**
+ * igt_hotplug_detected:
+ * @mon: A udev monitor initialized with #igt_watch_hotplug
+ * @timeout_secs: How long to wait for a hotplug event to occur.
+ *
+ * Assert that a hotplug event was received since we last checked the monitor.
+ *
+ * Returns: true if a sysfs hotplug event was received, false if we timed out
+ */
+bool igt_hotplug_detected(struct udev_monitor *mon, int timeout_secs)
+{
+	struct udev_device *dev;
+	const char *hotplug_val;
+	struct pollfd fd = {
+		.fd = udev_monitor_get_fd(mon),
+		.events = POLLIN
+	};
+	bool hotplug_received = false;
+
+	/* Go through all of the events pending on the udev monitor. Once we
+	 * receive a hotplug, we continue going through the rest of the events
+	 * so that redundant hotplug events don't change the results of future
+	 * checks
+	 */
+	while (!hotplug_received && poll(&fd, 1, timeout_secs * 1000)) {
+		dev = udev_monitor_receive_device(mon);
+
+		hotplug_val = udev_device_get_property_value(dev, "HOTPLUG");
+		if (hotplug_val && atoi(hotplug_val) == 1)
+			hotplug_received = true;
+
+		udev_device_unref(dev);
+	}
+
+	return hotplug_received;
+}
+
+/**
+ * igt_flush_hotplugs:
+ * @mon: A udev monitor initialized with #igt_watch_hotplug
+ *
+ * Get rid of any pending hotplug events
+ */
+void igt_flush_hotplugs(struct udev_monitor *mon)
+{
+	struct udev_device *dev;
+
+	while ((dev = udev_monitor_receive_device(mon)))
+		udev_device_unref(dev);
+}
+
+/**
+ * igt_cleanup_hotplug:
+ *
+ * Cleanup the resources allocated by #igt_watch_hotplug
+ */
+void igt_cleanup_hotplug(struct udev_monitor *mon)
+{
+	struct udev *udev = udev_monitor_get_udev(mon);
+
+	udev_monitor_unref(mon);
+	mon = NULL;
+	udev_unref(udev);
+}
+#endif
+
 /**
  * kmstest_get_vbl_flag:
  * @pipe_id: Pipe to convert to flag representation.
