@@ -56,6 +56,15 @@
 #include "igt_debugfs.h"
 #include "config.h"
 
+#ifdef HAVE_VALGRIND
+#include <valgrind/valgrind.h>
+#include <valgrind/memcheck.h>
+
+#define VG(x) x
+#else
+#define VG(x)
+#endif
+
 #include "ioctl_wrappers.h"
 
 /**
@@ -329,7 +338,7 @@ void gem_close(int fd, uint32_t handle)
 	do_ioctl(fd, DRM_IOCTL_GEM_CLOSE, &close_bo);
 }
 
-static int __gem_write(int fd, uint32_t handle, uint64_t offset, const void *buf, uint64_t length)
+int __gem_write(int fd, uint32_t handle, uint64_t offset, const void *buf, uint64_t length)
 {
 	struct drm_i915_gem_pwrite gem_pwrite;
 	int err;
@@ -338,7 +347,7 @@ static int __gem_write(int fd, uint32_t handle, uint64_t offset, const void *buf
 	gem_pwrite.handle = handle;
 	gem_pwrite.offset = offset;
 	gem_pwrite.size = length;
-	gem_pwrite.data_ptr = (uintptr_t)buf;
+	gem_pwrite.data_ptr = to_user_pointer(buf);
 
 	err = 0;
 	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_PWRITE, &gem_pwrite))
@@ -371,7 +380,7 @@ static int __gem_read(int fd, uint32_t handle, uint64_t offset, void *buf, uint6
 	gem_pread.handle = handle;
 	gem_pread.offset = offset;
 	gem_pread.size = length;
-	gem_pread.data_ptr = (uintptr_t)buf;
+	gem_pread.data_ptr = to_user_pointer(buf);
 
 	err = 0;
 	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_PREAD, &gem_pread))
@@ -394,29 +403,38 @@ void gem_read(int fd, uint32_t handle, uint64_t offset, void *buf, uint64_t leng
 	igt_assert_eq(__gem_read(fd, handle, offset, buf, length), 0);
 }
 
+int __gem_set_domain(int fd, uint32_t handle, uint32_t read, uint32_t write)
+{
+	struct drm_i915_gem_set_domain set_domain;
+	int err;
+
+	memset(&set_domain, 0, sizeof(set_domain));
+	set_domain.handle = handle;
+	set_domain.read_domains = read;
+	set_domain.write_domain = write;
+
+	err = 0;
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_SET_DOMAIN, &set_domain))
+		err = -errno;
+
+	return err;
+}
+
 /**
  * gem_set_domain:
  * @fd: open i915 drm file descriptor
  * @handle: gem buffer object handle
- * @read_domains: gem domain bits for read access
- * @write_domain: gem domain bit for write access
+ * @read: gem domain bits for read access
+ * @write: gem domain bit for write access
  *
  * This wraps the SET_DOMAIN ioctl, which is used to control the coherency of
  * the gem buffer object between the cpu and gtt mappings. It is also use to
  * synchronize with outstanding rendering in general, but for that use-case
  * please have a look at gem_sync().
  */
-void gem_set_domain(int fd, uint32_t handle,
-		    uint32_t read_domains, uint32_t write_domain)
+void gem_set_domain(int fd, uint32_t handle, uint32_t read, uint32_t write)
 {
-	struct drm_i915_gem_set_domain set_domain;
-
-	memset(&set_domain, 0, sizeof(set_domain));
-	set_domain.handle = handle;
-	set_domain.read_domains = read_domains;
-	set_domain.write_domain = write_domain;
-
-	do_ioctl(fd, DRM_IOCTL_I915_GEM_SET_DOMAIN, &set_domain);
+	igt_assert_eq(__gem_set_domain(fd, handle, read, write), 0);
 }
 
 /**
@@ -636,6 +654,8 @@ void *__gem_mmap__gtt(int fd, uint32_t handle, uint64_t size, unsigned prot)
 	else
 		errno = 0;
 
+	VG(VALGRIND_MAKE_MEM_DEFINED(ptr, size));
+
 	return ptr;
 }
 
@@ -657,6 +677,16 @@ void *gem_mmap__gtt(int fd, uint32_t handle, uint64_t size, unsigned prot)
 	return ptr;
 }
 
+int gem_munmap(void *ptr, uint64_t size)
+{
+	int ret = munmap(ptr, size);
+
+	if (ret == 0)
+		VG(VALGRIND_MAKE_MEM_NOACCESS(ptr, size));
+
+	return ret;
+}
+
 struct local_i915_gem_mmap_v2 {
 	uint32_t handle;
 	uint32_t pad;
@@ -734,8 +764,10 @@ void *__gem_mmap__wc(int fd, uint32_t handle, uint64_t offset, uint64_t size, un
 	if (igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_MMAP_v2, &arg))
 		return NULL;
 
+	VG(VALGRIND_MAKE_MEM_DEFINED(from_user_pointer(arg.addr_ptr), arg.size));
+
 	errno = 0;
-	return (void *)(uintptr_t)arg.addr_ptr;
+	return from_user_pointer(arg.addr_ptr);
 }
 
 /**
@@ -781,8 +813,10 @@ void *__gem_mmap__cpu(int fd, uint32_t handle, uint64_t offset, uint64_t size, u
 	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP, &mmap_arg))
 		return NULL;
 
+	VG(VALGRIND_MAKE_MEM_DEFINED(from_user_pointer(mmap_arg.addr_ptr), mmap_arg.size));
+
 	errno = 0;
-	return (void *)(uintptr_t)mmap_arg.addr_ptr;
+	return from_user_pointer(mmap_arg.addr_ptr);
 }
 
 /**
@@ -997,7 +1031,7 @@ int __gem_userptr(int fd, void *ptr, int size, int read_only, uint32_t flags, ui
 	struct local_i915_gem_userptr userptr;
 
 	memset(&userptr, 0, sizeof(userptr));
-	userptr.user_ptr = (uintptr_t)ptr;
+	userptr.user_ptr = to_user_pointer(ptr);
 	userptr.user_size = size;
 	userptr.flags = flags;
 	if (read_only)
@@ -1433,6 +1467,35 @@ bool gem_has_softpin(int fd)
 	return has_softpin;
 }
 
+#define LOCAL_PARAM_HAS_EXEC_FENCE 44
+/**
+ * gem_has_exec_fence:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to query whether in/out fence support in execbuffer is
+ * available.
+ *
+ * Returns: Whether fence support is available
+ */
+bool gem_has_exec_fence(int fd)
+{
+	static int has_exec_fence = -1;
+
+	if (has_exec_fence < 0) {
+		struct drm_i915_getparam gp;
+
+		memset(&gp, 0, sizeof(gp));
+		gp.param = LOCAL_PARAM_HAS_EXEC_FENCE;
+		gp.value = &has_exec_fence;
+
+		has_exec_fence = 0;
+		ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp));
+		errno = 0;
+	}
+
+	return has_exec_fence;
+}
+
 /**
  * gem_require_caching:
  * @fd: open i915 drm file descriptor
@@ -1451,6 +1514,17 @@ void gem_require_caching(int fd)
 	errno = 0;
 }
 
+void igt_require_gem(int fd)
+{
+	int err;
+
+	err = 0;
+	if (ioctl(fd, DRM_IOCTL_I915_GEM_THROTTLE))
+		err = -errno;
+
+	igt_require_f(err == 0, "Unresponsive i915/GEM device\n");
+}
+
 bool gem_has_ring(int fd, unsigned ring)
 {
 	struct drm_i915_gem_execbuffer2 execbuf;
@@ -1464,7 +1538,7 @@ bool gem_has_ring(int fd, unsigned ring)
 
 	memset(&exec, 0, sizeof(exec));
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&exec;
+	execbuf.buffers_ptr = to_user_pointer(&exec);
 	execbuf.buffer_count = 1;
 	execbuf.flags = ring;
 	return __gem_execbuf(fd, &execbuf) == -ENOENT;
