@@ -115,7 +115,7 @@ static void run_on_ring(int fd, unsigned ring_id, const char *ring_name)
 	obj[0].batch[0] = MI_BATCH_BUFFER_END;
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&execobj;
+	execbuf.buffers_ptr = to_user_pointer(&execobj);
 	execbuf.buffer_count = 1;
 	execbuf.flags = ring_id;
 
@@ -145,8 +145,10 @@ igt_main
 
 	igt_skip_on_simulation();
 
-	igt_fixture
+	igt_fixture {
 		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
+	}
 
 	for (e = intel_execution_engines; e->name; e++)
 		igt_subtest_f("%s%s", e->exec_id ? "" : "basic-", e->name)
