@@ -341,13 +341,60 @@ test_coherency(int fd)
 	gem_close(fd, handle);
 }
 
+static void
+test_hang(int fd)
+{
+	igt_hang_t hang;
+	uint32_t patterns[] = {
+		0, 0xaaaaaaaa, 0x55555555, 0xcccccccc,
+	};
+	uint32_t *gtt[3];
+	int last_pattern = 0;
+	int next_pattern = 1;
+	int i;
+
+	for (i = I915_TILING_NONE; i <= I915_TILING_Y; i++) {
+		uint32_t handle;
+
+		handle = gem_create(fd, OBJECT_SIZE);
+		gem_set_tiling(fd, handle, i, 2048);
+
+		gtt[i] = gem_mmap__gtt(fd, handle, OBJECT_SIZE, PROT_WRITE);
+		set_domain_gtt(fd, handle);
+		gem_close(fd, handle);
+	}
+
+	hang = igt_hang_ring(fd, I915_EXEC_RENDER);
+
+	do {
+		for (i = 0; i < OBJECT_SIZE / 64; i++) {
+			int x = 16*i + (i%16);
+
+			igt_assert(gtt[0][x] == patterns[last_pattern]);
+			igt_assert(gtt[1][x] == patterns[last_pattern]);
+			igt_assert(gtt[2][x] == patterns[last_pattern]);
+
+			gtt[0][x] = patterns[next_pattern];
+			gtt[1][x] = patterns[next_pattern];
+			gtt[2][x] = patterns[next_pattern];
+		}
+
+		last_pattern = next_pattern;
+		next_pattern = (next_pattern + 1) % ARRAY_SIZE(patterns);
+	} while (gem_bo_busy(fd, hang.handle));
+
+	igt_post_hang_ring(fd, hang);
+
+	munmap(gtt[0], OBJECT_SIZE);
+	munmap(gtt[1], OBJECT_SIZE);
+	munmap(gtt[2], OBJECT_SIZE);
+}
+
 static int min_tile_width(uint32_t devid, int tiling)
 {
 	if (tiling < 0) {
-		tiling = -tiling;
-
 		if (intel_gen(devid) >= 4)
-			return 4096 - min_tile_width(devid, tiling);
+			return 4096 - min_tile_width(devid, -tiling);
 		else
 			return 1024;
 
@@ -366,10 +413,8 @@ static int min_tile_width(uint32_t devid, int tiling)
 static int max_tile_width(uint32_t devid, int tiling)
 {
 	if (tiling < 0) {
-		tiling = -tiling;
-
 		if (intel_gen(devid) >= 4)
-			return 4096 + min_tile_width(devid, tiling);
+			return 4096 + min_tile_width(devid, -tiling);
 		else
 			return 2048;
 	}
@@ -506,6 +551,16 @@ static void copy_wc_page(void *dst, const void *src)
 }
 #endif
 
+static unsigned int tile_row_size(int tiling, unsigned int stride)
+{
+	if (tiling < 0)
+		tiling = -tiling;
+
+	return stride * (tiling == I915_TILING_Y ? 32 : 8);
+}
+
+#define rounddown(x, y) (x - (x%y))
+
 static void
 test_huge_copy(int fd, int huge, int tiling_a, int tiling_b, int ncpus)
 {
@@ -534,38 +589,44 @@ test_huge_copy(int fd, int huge, int tiling_a, int tiling_b, int ncpus)
 	intel_require_memory(2*ncpus, huge_object_size, mode);
 
 	igt_fork(child, ncpus) {
+		uint64_t valid_size = huge_object_size;
 		uint32_t bo;
 		char *a, *b;
 
 		bo = gem_create(fd, huge_object_size);
-		if (tiling_a)
+		if (tiling_a) {
 			igt_require(__gem_set_tiling(fd, bo, abs(tiling_a), min_tile_width(devid, tiling_a)) == 0);
+			valid_size = rounddown(valid_size, tile_row_size(tiling_a, min_tile_width(devid, tiling_a)));
+		}
 		a = __gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
 		igt_require(a);
 		gem_close(fd, bo);
 
-		for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
-			uint32_t *ptr = (uint32_t *)(a + PAGE_SIZE*i);
-			for (int j = 0; j < PAGE_SIZE/4; j++)
-				ptr[j] = i + j;
-			igt_progress("Writing a ", i, huge_object_size / PAGE_SIZE);
-		}
-
 		bo = gem_create(fd, huge_object_size);
-		if (tiling_b)
+		if (tiling_b) {
 			igt_require(__gem_set_tiling(fd, bo, abs(tiling_b), max_tile_width(devid, tiling_b)) == 0);
+			valid_size = rounddown(valid_size, tile_row_size(tiling_b, max_tile_width(devid, tiling_b)));
+		}
 		b = __gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
 		igt_require(b);
 		gem_close(fd, bo);
 
-		for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+		for (i = 0; i < valid_size / PAGE_SIZE; i++) {
+			uint32_t *ptr = (uint32_t *)(a + PAGE_SIZE*i);
+			for (int j = 0; j < PAGE_SIZE/4; j++)
+				ptr[j] = i + j;
+			igt_progress("Writing a ", i, valid_size / PAGE_SIZE);
+		}
+
+
+		for (i = 0; i < valid_size / PAGE_SIZE; i++) {
 			uint32_t *ptr = (uint32_t *)(b + PAGE_SIZE*i);
 			for (int j = 0; j < PAGE_SIZE/4; j++)
 				ptr[j] = ~(i + j);
-			igt_progress("Writing b ", i, huge_object_size / PAGE_SIZE);
+			igt_progress("Writing b ", i, valid_size / PAGE_SIZE);
 		}
 
-		for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+		for (i = 0; i < valid_size / PAGE_SIZE; i++) {
 			uint32_t *A = (uint32_t *)(a + PAGE_SIZE*i);
 			uint32_t *B = (uint32_t *)(b + PAGE_SIZE*i);
 			uint32_t A_tmp[PAGE_SIZE/sizeof(uint32_t)];
@@ -581,10 +642,10 @@ test_huge_copy(int fd, int huge, int tiling_a, int tiling_b, int ncpus)
 			memcpy(A, A_tmp, PAGE_SIZE);
 			memcpy(B, B_tmp, PAGE_SIZE);
 
-			igt_progress("Copying a<->b ", i, huge_object_size / PAGE_SIZE);
+			igt_progress("Copying a<->b ", i, valid_size / PAGE_SIZE);
 		}
 
-		for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+		for (i = 0; i < valid_size / PAGE_SIZE; i++) {
 			uint32_t page[PAGE_SIZE/sizeof(uint32_t)];
 			copy_wc_page(page, a + PAGE_SIZE*i);
 			for (int j = 0; j < PAGE_SIZE/sizeof(uint32_t); j++)
@@ -592,11 +653,11 @@ test_huge_copy(int fd, int huge, int tiling_a, int tiling_b, int ncpus)
 					igt_assert_eq_u32(page[j], ~(i + j));
 				else
 					igt_assert_eq_u32(page[j], i + j);
-			igt_progress("Checking a ", i, huge_object_size / PAGE_SIZE);
+			igt_progress("Checking a ", i, valid_size / PAGE_SIZE);
 		}
 		munmap(a, huge_object_size);
 
-		for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+		for (i = 0; i < valid_size / PAGE_SIZE; i++) {
 			uint32_t page[PAGE_SIZE/sizeof(uint32_t)];
 			copy_wc_page(page, b + PAGE_SIZE*i);
 			for (int j = 0; j < PAGE_SIZE/sizeof(uint32_t); j++)
@@ -604,7 +665,7 @@ test_huge_copy(int fd, int huge, int tiling_a, int tiling_b, int ncpus)
 					igt_assert_eq_u32(page[j], ~(i + j));
 				else
 					igt_assert_eq_u32(page[j], i + j);
-			igt_progress("Checking b ", i, huge_object_size / PAGE_SIZE);
+			igt_progress("Checking b ", i, valid_size / PAGE_SIZE);
 		}
 		munmap(b, huge_object_size);
 	}
@@ -731,6 +792,8 @@ igt_main
 		test_write_gtt(fd);
 	igt_subtest("coherency")
 		test_coherency(fd);
+	igt_subtest("hang")
+		test_hang(fd);
 	igt_subtest("basic-read-write")
 		test_read_write(fd, READ_BEFORE_WRITE);
 	igt_subtest("basic-write-read")
