@@ -92,7 +92,7 @@ static void cursor_enable(data_t *data)
 	igt_output_t *output = data->output;
 	igt_plane_t *cursor;
 
-	cursor = igt_output_get_plane(output, IGT_PLANE_CURSOR);
+	cursor = igt_output_get_plane_type(output, DRM_PLANE_TYPE_CURSOR);
 	igt_plane_set_fb(cursor, &data->fb);
 	igt_plane_set_size(cursor, data->curw, data->curh);
 }
@@ -102,7 +102,7 @@ static void cursor_disable(data_t *data)
 	igt_output_t *output = data->output;
 	igt_plane_t *cursor;
 
-	cursor = igt_output_get_plane(output, IGT_PLANE_CURSOR);
+	cursor = igt_output_get_plane_type(output, DRM_PLANE_TYPE_CURSOR);
 	igt_plane_set_fb(cursor, NULL);
 }
 
@@ -120,7 +120,7 @@ static void do_single_test(data_t *data, int x, int y)
 	/* Hardware test */
 	igt_paint_test_pattern(cr, data->screenw, data->screenh);
 	cursor_enable(data);
-	cursor = igt_output_get_plane(data->output, IGT_PLANE_CURSOR);
+	cursor = igt_output_get_plane_type(data->output, DRM_PLANE_TYPE_CURSOR);
 	igt_plane_set_position(cursor, x, y);
 	igt_display_commit(display);
 	igt_wait_for_vblank(data->drm_fd, data->pipe);
@@ -140,7 +140,8 @@ static void do_single_test(data_t *data, int x, int y)
 		}
 
 		if (data->flags & TEST_SUSPEND)
-			igt_system_suspend_autoresume();
+			igt_system_suspend_autoresume(SUSPEND_STATE_MEM,
+						      SUSPEND_TEST_NONE);
 
 		igt_pipe_crc_collect_crc(pipe_crc, &crc_after);
 		igt_assert_crc_equal(&crc, &crc_after);
@@ -173,7 +174,7 @@ static void do_fail_test(data_t *data, int x, int y, int expect)
 	/* Hardware test */
 	igt_paint_test_pattern(cr, data->screenw, data->screenh);
 	cursor_enable(data);
-	cursor = igt_output_get_plane(data->output, IGT_PLANE_CURSOR);
+	cursor = igt_output_get_plane_type(data->output, DRM_PLANE_TYPE_CURSOR);
 	igt_plane_set_position(cursor, x, y);
 	ret = igt_display_try_commit2(display, COMMIT_LEGACY);
 
@@ -281,7 +282,7 @@ static void test_crc_random(data_t *data)
 	}
 }
 
-static bool prepare_crtc(data_t *data, igt_output_t *output,
+static void prepare_crtc(data_t *data, igt_output_t *output,
 			 int cursor_w, int cursor_h)
 {
 	drmModeModeInfo *mode;
@@ -291,13 +292,6 @@ static bool prepare_crtc(data_t *data, igt_output_t *output,
 	/* select the pipe we want to use */
 	igt_output_set_pipe(output, data->pipe);
 	cursor_disable(data);
-	igt_display_commit(display);
-
-	if (!output->valid) {
-		igt_output_set_pipe(output, PIPE_ANY);
-		igt_display_commit(display);
-		return false;
-	}
 
 	/* create and set the primary plane fb */
 	mode = igt_output_get_mode(output);
@@ -307,7 +301,7 @@ static bool prepare_crtc(data_t *data, igt_output_t *output,
 			    0.0, 0.0, 0.0,
 			    &data->primary_fb);
 
-	primary = igt_output_get_plane(output, IGT_PLANE_PRIMARY);
+	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
 	igt_plane_set_fb(primary, &data->primary_fb);
 
 	igt_display_commit(display);
@@ -336,8 +330,6 @@ static bool prepare_crtc(data_t *data, igt_output_t *output,
 
 	/* get reference crc w/o cursor */
 	igt_pipe_crc_collect_crc(data->pipe_crc, &data->ref_crc);
-
-	return true;
 }
 
 static void cleanup_crtc(data_t *data, igt_output_t *output)
@@ -350,7 +342,7 @@ static void cleanup_crtc(data_t *data, igt_output_t *output)
 
 	igt_remove_fb(data->drm_fd, &data->primary_fb);
 
-	primary = igt_output_get_plane(output, IGT_PLANE_PRIMARY);
+	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
 	igt_plane_set_fb(primary, NULL);
 
 	igt_output_set_pipe(output, PIPE_ANY);
@@ -367,31 +359,28 @@ static void run_test(data_t *data, void (*testfunc)(data_t *), int cursor_w, int
 	igt_require(cursor_w <= data->cursor_max_w &&
 		    cursor_h <= data->cursor_max_h);
 
-	for_each_connected_output(display, output) {
+	for_each_pipe_with_valid_output(display, p, output) {
 		data->output = output;
-		for_each_pipe(display, p) {
-			data->pipe = p;
+		data->pipe = p;
 
-			if (!prepare_crtc(data, output, cursor_w, cursor_h))
-				continue;
+		prepare_crtc(data, output, cursor_w, cursor_h);
 
-			valid_tests++;
+		valid_tests++;
 
-			igt_info("Beginning %s on pipe %s, connector %s\n",
-				 igt_subtest_name(),
-				 kmstest_pipe_name(data->pipe),
-				 igt_output_name(output));
+		igt_info("Beginning %s on pipe %s, connector %s\n",
+			  igt_subtest_name(),
+			  kmstest_pipe_name(data->pipe),
+			  igt_output_name(output));
 
-			testfunc(data);
+		testfunc(data);
 
-			igt_info("\n%s on pipe %s, connector %s: PASSED\n\n",
-				 igt_subtest_name(),
-				 kmstest_pipe_name(data->pipe),
-				 igt_output_name(output));
+		igt_info("\n%s on pipe %s, connector %s: PASSED\n\n",
+			  igt_subtest_name(),
+			  kmstest_pipe_name(data->pipe),
+			  igt_output_name(output));
 
-			/* cleanup what prepare_crtc() has done */
-			cleanup_crtc(data, output);
-		}
+		/* cleanup what prepare_crtc() has done */
+		cleanup_crtc(data, output);
 	}
 
 	igt_require_f(valid_tests, "no valid crtc/connector combinations found\n");
