@@ -166,14 +166,12 @@ struct {
 struct {
 	bool can_test;
 
-	bool supports_compressing;
 	bool supports_last_action;
 
 	struct timespec last_action;
 } fbc = {
 	.can_test = false,
 	.supports_last_action = false,
-	.supports_compressing = false,
 };
 
 struct {
@@ -203,9 +201,11 @@ struct both_crcs *wanted_crc;
 struct {
 	int fd;
 	bool supported;
+	bool reliable;
 } sink_crc = {
 	.fd = -1,
-	.supported = false,
+	.supported = true,
+	.reliable = true,
 };
 
 /* The goal of this structure is to easily allow us to deal with cases where we
@@ -377,7 +377,7 @@ static void print_mode_info(const char *screen, struct modeset_params *params)
 		 screen,
 		 kmstest_connector_type_str(c->connector_type),
 		 params->mode->name,
-		 params->crtc_id);
+		 kmstest_get_crtc_idx(drm.res, params->crtc_id));
 }
 
 static void init_mode_params(struct modeset_params *params, uint32_t crtc_id,
@@ -894,18 +894,6 @@ static bool fbc_wait_for_compression(void)
 	return igt_wait(fbc_is_compressing(), 2000, 1);
 }
 
-static void fbc_setup_compressing(void)
-{
-	char buf[128];
-
-	igt_debugfs_read("i915_fbc_status", buf);
-
-	if (strstr(buf, "\nCompressing:"))
-		fbc.supports_compressing = true;
-	else
-		igt_info("FBC compression information not supported\n");
-}
-
 static bool fbc_not_enough_stolen(void)
 {
 	char buf[128];
@@ -914,6 +902,14 @@ static bool fbc_not_enough_stolen(void)
 	return strstr(buf, "FBC disabled: not enough stolen memory\n");
 }
 
+static bool fbc_stride_not_supported(void)
+{
+	char buf[128];
+
+	igt_debugfs_read("i915_fbc_status", buf);
+	return strstr(buf, "FBC disabled: framebuffer stride not supported\n");
+}
+
 static bool fbc_wait_until_enabled(void)
 {
 	return igt_wait(fbc_is_enabled(), 2000, 1);
@@ -933,18 +929,31 @@ static void get_sink_crc(sink_crc_t *crc, bool mandatory)
 {
 	int rc, errno_;
 
+	if (!sink_crc.supported) {
+		memcpy(crc, "unsupported!", SINK_CRC_SIZE);
+		return;
+	}
+
 	lseek(sink_crc.fd, 0, SEEK_SET);
 
 	rc = read(sink_crc.fd, crc->data, SINK_CRC_SIZE);
 	errno_ = errno;
 
-	if (rc == -1 && errno_ == ETIMEDOUT) {
+	if (rc == -1 && errno_ == ENOTTY) {
+		igt_info("Sink CRC not supported: panel doesn't support it\n");
+		sink_crc.supported = false;
+	} else if (rc == -1 && errno_ == ETIMEDOUT) {
+		if (sink_crc.reliable) {
+			igt_info("Sink CRC is unreliable on this machine.\n");
+			sink_crc.reliable = false;
+		}
+
 		if (mandatory)
-			igt_skip("Sink CRC is unreliable on this machine. Try running this test again individually\n");
-		else
-			igt_info("Sink CRC is unreliable on this machine. Try running this test again individually\n");
+			igt_skip("Sink CRC is unreliable on this machine.\n");
+	} else {
+		igt_assert_f(rc != -1, "Unexpected error: %d\n", errno_);
+		igt_assert(rc == SINK_CRC_SIZE);
 	}
-	igt_assert(rc == SINK_CRC_SIZE);
 }
 
 static bool sink_crc_equal(sink_crc_t *a, sink_crc_t *b)
@@ -1219,11 +1228,7 @@ static void print_crc(const char *str, struct both_crcs *crc)
 static void collect_crcs(struct both_crcs *crcs, bool mandatory_sink_crc)
 {
 	igt_pipe_crc_collect_crc(pipe_crc, &crcs->pipe);
-
-	if (sink_crc.supported)
-		get_sink_crc(&crcs->sink, mandatory_sink_crc);
-	else
-		memcpy(&crcs->sink, "unsupported!", SINK_CRC_SIZE);
+	get_sink_crc(&crcs->sink, mandatory_sink_crc);
 }
 
 static void init_blue_crc(enum pixel_format format, bool mandatory_sink_crc)
@@ -1389,19 +1394,21 @@ static void setup_modeset(void)
 
 static void teardown_modeset(void)
 {
-	destroy_fbs(FORMAT_DEFAULT);
+	enum pixel_format f;
+
+	for (f = 0; f < FORMAT_COUNT; f++)
+		destroy_fbs(f);
 }
 
 static void setup_sink_crc(void)
 {
-	ssize_t rc;
 	sink_crc_t crc;
-	int errno_;
 	drmModeConnectorPtr c;
 
 	c = get_connector(prim_mode_params.connector_id);
 	if (c->connector_type != DRM_MODE_CONNECTOR_eDP) {
 		igt_info("Sink CRC not supported: primary screen is not eDP\n");
+		sink_crc.supported = false;
 		return;
 	}
 
@@ -1416,17 +1423,8 @@ static void setup_sink_crc(void)
 	sink_crc.fd = igt_debugfs_open("i915_sink_crc_eDP1", O_RDONLY);
 	igt_assert_lte(0, sink_crc.fd);
 
-	rc = read(sink_crc.fd, crc.data, SINK_CRC_SIZE);
-	errno_ = errno;
-	if (rc == -1 && errno_ == ENOTTY)
-		igt_info("Sink CRC not supported: panel doesn't support it\n");
-	if (rc == -1 && errno_ == ETIMEDOUT)
-		igt_info("Sink CRC not reliable on this panel: skipping it\n");
-	else if (rc == SINK_CRC_SIZE)
-		sink_crc.supported = true;
-	else
-		igt_info("Unexpected sink CRC error, rc=:%zd errno:%d %s\n",
-			 rc, errno_, strerror(errno_));
+	/* Do a first read to try to detect if it's supported. */
+	get_sink_crc(&crc, false);
 }
 
 static void setup_crcs(void)
@@ -1525,7 +1523,6 @@ static void setup_fbc(void)
 	fbc.can_test = true;
 
 	fbc_setup_last_action();
-	fbc_setup_compressing();
 }
 
 static void teardown_fbc(void)
@@ -1675,9 +1672,9 @@ static int adjust_assertion_flags(const struct test_mode *t, int flags)
 	igt_assert_crc_equal(&crc_.pipe, &wanted_crc->pipe);		\
 	if (mandatory_sink_crc)						\
 		assert_sink_crc_equal(&crc_.sink, &wanted_crc->sink);	\
-	else								\
-		if (!sink_crc_equal(&crc_.sink, &wanted_crc->sink))	\
-			igt_info("Sink CRC differ, but not required\n"); \
+	else if (sink_crc.reliable &&					\
+		 !sink_crc_equal(&crc_.sink, &wanted_crc->sink))	\
+		igt_info("Sink CRC differ, but not required\n"); 	\
 } while (0)
 
 #define do_status_assertions(flags_) do {				\
@@ -1689,13 +1686,13 @@ static int adjust_assertion_flags(const struct test_mode *t, int flags)
 									\
 	if (flags_ & ASSERT_FBC_ENABLED) {				\
 		igt_require(!fbc_not_enough_stolen());			\
+		igt_require(!fbc_stride_not_supported());		\
 		if (!fbc_wait_until_enabled()) {			\
 			fbc_print_status();				\
 			igt_assert_f(false, "FBC disabled\n");		\
 		}							\
 									\
-		if (fbc.supports_compressing && 			\
-		    opt.fbc_check_compression)				\
+		if (opt.fbc_check_compression)				\
 			igt_assert(fbc_wait_for_compression());		\
 	} else if (flags_ & ASSERT_FBC_DISABLED) {			\
 		igt_assert(!fbc_wait_until_enabled());			\
@@ -3027,11 +3024,14 @@ static void badstride_subtest(const struct test_mode *t)
 	set_mode_for_params(params);
 	do_assertions(0);
 
-	/* We can't use the page flip IOCTL to flip to a buffer with a different
-	 * stride. */
+	/*
+	 * We previously couldn't use the page flip IOCTL to flip to a buffer
+	 * with a different stride. With the atomic page flip helper we can,
+	 * so allow page flip to fail and succeed.
+	 */
 	rc = drmModePageFlip(drm.fd, params->crtc_id, wide_fb.fb_id, 0, NULL);
-	igt_assert(rc == -EINVAL);
-	do_assertions(0);
+	igt_assert(rc == -EINVAL || rc == 0);
+	do_assertions(rc == 0 ? ASSERT_FBC_DISABLED : 0);
 
 	igt_remove_fb(drm.fd, &wide_fb);
 }
@@ -3092,9 +3092,12 @@ static void stridechange_subtest(const struct test_mode *t)
 	set_prim_plane_for_params(params);
 	do_assertions(0);
 
-	/* We just can't page flip with a new stride. */
+	/*
+	 * Try to set a new stride. with the page flip api. This is allowed
+	 * with the atomic page flip helper, but not with the legacy page flip.
+	 */
 	rc = drmModePageFlip(drm.fd, params->crtc_id, new_fb.fb_id, 0, NULL);
-	igt_assert(rc == -EINVAL);
+	igt_assert(rc == -EINVAL || rc == 0);
 	do_assertions(0);
 
 	igt_remove_fb(drm.fd, &new_fb);
