@@ -100,7 +100,7 @@ static void test(data_t *data)
 	igt_assert_crc_equal(&crc, &data->ref_crc);
 }
 
-static bool prepare_crtc(data_t *data)
+static void prepare_crtc(data_t *data)
 {
 	igt_display_t *display = &data->display;
 	igt_output_t *output = data->output;
@@ -108,13 +108,6 @@ static bool prepare_crtc(data_t *data)
 
 	/* select the pipe we want to use */
 	igt_output_set_pipe(output, data->pipe);
-	igt_display_commit(display);
-
-	if (!output->valid) {
-		igt_output_set_pipe(output, PIPE_ANY);
-		igt_display_commit(display);
-		return false;
-	}
 
 	mode = igt_output_get_mode(output);
 
@@ -123,7 +116,7 @@ static bool prepare_crtc(data_t *data)
 			    DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE,
 			    1.0, 1.0, 1.0, &data->fb[0]);
 
-	data->primary = igt_output_get_plane(output, IGT_PLANE_PRIMARY);
+	data->primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
 
 	igt_plane_set_fb(data->primary, &data->fb[0]);
 	igt_display_commit(display);
@@ -136,8 +129,6 @@ static bool prepare_crtc(data_t *data)
 
 	/* get reference crc for the white fb */
 	igt_pipe_crc_collect_crc(data->pipe_crc, &data->ref_crc);
-
-	return true;
 }
 
 static void cleanup_crtc(data_t *data)
@@ -164,20 +155,16 @@ static void run_test(data_t *data)
 	igt_output_t *output;
 	enum pipe pipe;
 
-	for_each_connected_output(display, output) {
+	for_each_pipe_with_valid_output(display, pipe, output) {
 		data->output = output;
-		for_each_pipe(display, pipe) {
-			data->pipe = pipe;
+		data->pipe = pipe;
 
-			if (!prepare_crtc(data))
-				continue;
+		prepare_crtc(data);
+		test(data);
+		cleanup_crtc(data);
 
-			test(data);
-			cleanup_crtc(data);
-
-			/* once is enough */
-			return;
-		}
+		/* once is enough */
+		return;
 	}
 
 	igt_skip("no valid crtc/connector combinations found\n");
