@@ -97,23 +97,29 @@ static void commit_crtc(data_t *data, igt_output_t *output, igt_plane_t *plane)
 	 * we create an fb covering the crtc and call commit
 	 */
 
-	primary = igt_output_get_plane(output, IGT_PLANE_PRIMARY);
-	igt_plane_set_fb(primary, &data->fb_modeset);
-	igt_display_commit(display);
-
-	igt_plane_set_fb(plane, &data->fb);
-
-	if (!plane->is_cursor)
-		igt_plane_set_position(plane, data->pos_x, data->pos_y);
-
-	if (plane->is_primary || plane->is_cursor) {
-		igt_require(data->display.has_universal_planes);
+	if (plane->type == DRM_PLANE_TYPE_PRIMARY ||
+	    plane->type == DRM_PLANE_TYPE_CURSOR)
 		commit = COMMIT_UNIVERSAL;
-	}
 
 	if (data->display.is_atomic)
 		commit = COMMIT_ATOMIC;
 
+	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
+
+	if (commit < COMMIT_ATOMIC) {
+		igt_plane_set_fb(primary, &data->fb_modeset);
+		primary->rotation_changed = false;
+		igt_display_commit(display);
+
+		if (plane->type == DRM_PLANE_TYPE_PRIMARY)
+			primary->rotation_changed = true;
+	}
+
+	igt_plane_set_fb(plane, &data->fb);
+
+	if (plane->type != DRM_PLANE_TYPE_CURSOR)
+		igt_plane_set_position(plane, data->pos_x, data->pos_y);
+
 	igt_display_commit2(display, commit);
 }
 
@@ -129,6 +135,7 @@ static void prepare_crtc(data_t *data, igt_output_t *output, enum pipe pipe,
 				data->override_fmt : DRM_FORMAT_XRGB8888;
 
 	igt_output_set_pipe(output, pipe);
+	igt_plane_set_rotation(plane, IGT_ROTATION_0);
 
 	/* create the pipe_crc object for this pipe */
 	igt_pipe_crc_free(data->pipe_crc);
@@ -154,8 +161,8 @@ static void prepare_crtc(data_t *data, igt_output_t *output, enum pipe pipe,
 	    data->rotation == IGT_ROTATION_270) {
 		tiling = data->override_tiling ?
 			 data->override_tiling : LOCAL_I915_FORMAT_MOD_Y_TILED;
-		w = h =  mode->vdisplay;
-	} else if (plane->is_cursor) {
+		w = h = min(mode->hdisplay, mode->vdisplay);
+	} else if (plane->type == DRM_PLANE_TYPE_CURSOR) {
 		pixel_format = data->override_fmt ?
 			       data->override_fmt : DRM_FORMAT_ARGB8888;
 		w = h = 128;
@@ -207,14 +214,18 @@ static void cleanup_crtc(data_t *data, igt_output_t *output, igt_plane_t *plane)
 		igt_remove_fb(data->gfx_fd, &data->fb_flip);
 
 	/* XXX: see the note in prepare_crtc() */
-	if (!plane->is_primary) {
+	if (plane->type != DRM_PLANE_TYPE_PRIMARY) {
 		igt_plane_t *primary;
 
-		primary = igt_output_get_plane(output, IGT_PLANE_PRIMARY);
+		primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
 		igt_plane_set_fb(primary, NULL);
 	}
 
 	igt_plane_set_fb(plane, NULL);
+	igt_plane_set_rotation(plane, IGT_ROTATION_0);
+
+	igt_display_commit2(display, COMMIT_UNIVERSAL);
+
 	igt_output_set_pipe(output, PIPE_ANY);
 
 	igt_display_commit(display);
@@ -237,7 +248,7 @@ static void wait_for_pageflip(int fd)
 	igt_assert(drmHandleEvent(fd, &evctx) == 0);
 }
 
-static void test_plane_rotation(data_t *data, enum igt_plane plane_type)
+static void test_plane_rotation(data_t *data, int plane_type)
 {
 	igt_display_t *display = &data->display;
 	igt_output_t *output;
@@ -248,80 +259,69 @@ static void test_plane_rotation(data_t *data, enum igt_plane plane_type)
 	unsigned int flip_count;
 	int ret;
 
-	if (plane_type == IGT_PLANE_PRIMARY || plane_type == IGT_PLANE_CURSOR) {
-		igt_require(data->display.has_universal_planes);
+	if (plane_type == DRM_PLANE_TYPE_PRIMARY || plane_type == DRM_PLANE_TYPE_CURSOR)
 		commit = COMMIT_UNIVERSAL;
-	}
+
+	if (plane_type == DRM_PLANE_TYPE_CURSOR)
+		igt_require(display->has_cursor_plane);
 
 	if (data->display.is_atomic)
 		commit = COMMIT_ATOMIC;
 
-	for_each_connected_output(display, output) {
-		for_each_pipe(display, pipe) {
-			igt_plane_t *plane;
+	for_each_pipe_with_valid_output(display, pipe, output) {
+		igt_plane_t *plane;
 
-			igt_output_set_pipe(output, pipe);
+		igt_output_set_pipe(output, pipe);
 
-			plane = igt_output_get_plane(output, plane_type);
-			igt_require(igt_plane_supports_rotation(plane));
+		plane = igt_output_get_plane_type(output, plane_type);
+		igt_require(igt_plane_supports_rotation(plane));
 
-			prepare_crtc(data, output, pipe, plane);
+		prepare_crtc(data, output, pipe, plane);
 
-			igt_display_commit2(display, commit);
+		igt_display_commit2(display, commit);
 
-			/* collect unrotated CRC */
-			igt_pipe_crc_collect_crc(data->pipe_crc, &crc_unrotated);
+		/* collect unrotated CRC */
+		igt_pipe_crc_collect_crc(data->pipe_crc, &crc_unrotated);
 
-			igt_plane_set_rotation(plane, data->rotation);
-			ret = igt_display_try_commit2(display, commit);
-			if (data->override_fmt || data->override_tiling) {
-				igt_assert_eq(ret, -EINVAL);
-			} else {
-				igt_assert_eq(ret, 0);
-				igt_pipe_crc_collect_crc(data->pipe_crc,
-							 &crc_output);
-				igt_assert_crc_equal(&data->ref_crc,
-						     &crc_output);
-			}
-
-			flip_count = data->flip_stress;
-			while (flip_count--) {
-				ret = drmModePageFlip(data->gfx_fd,
-						      output->config.crtc->crtc_id,
-						      data->fb_flip.fb_id,
-						      DRM_MODE_PAGE_FLIP_EVENT,
-						      NULL);
-				igt_assert(ret == 0);
-				wait_for_pageflip(data->gfx_fd);
-				ret = drmModePageFlip(data->gfx_fd,
-						      output->config.crtc->crtc_id,
-						      data->fb.fb_id,
-						      DRM_MODE_PAGE_FLIP_EVENT,
-						      NULL);
-				igt_assert(ret == 0);
-				wait_for_pageflip(data->gfx_fd);
-			}
-
-			/*
-			 * check the rotation state has been reset when the VT
-			 * mode is restored
-			 */
-			kmstest_restore_vt_mode();
-			kmstest_set_vt_graphics_mode();
-
-			commit_crtc(data, output, plane);
-
-			igt_pipe_crc_collect_crc(data->pipe_crc, &crc_output);
-			igt_assert_crc_equal(&crc_unrotated, &crc_output);
-
-			valid_tests++;
-			cleanup_crtc(data, output, plane);
+		igt_plane_set_rotation(plane, data->rotation);
+		ret = igt_display_try_commit2(display, commit);
+		if (data->override_fmt || data->override_tiling) {
+			igt_assert_eq(ret, -EINVAL);
+		} else {
+			igt_assert_eq(ret, 0);
+			igt_pipe_crc_collect_crc(data->pipe_crc,
+						  &crc_output);
+			igt_assert_crc_equal(&data->ref_crc,
+					      &crc_output);
 		}
+
+		flip_count = data->flip_stress;
+		while (flip_count--) {
+			ret = drmModePageFlip(data->gfx_fd,
+					      output->config.crtc->crtc_id,
+					      data->fb_flip.fb_id,
+					      DRM_MODE_PAGE_FLIP_EVENT,
+					      NULL);
+			igt_assert_eq(ret, 0);
+			wait_for_pageflip(data->gfx_fd);
+			ret = drmModePageFlip(data->gfx_fd,
+					      output->config.crtc->crtc_id,
+					      data->fb.fb_id,
+					      DRM_MODE_PAGE_FLIP_EVENT,
+					      NULL);
+			igt_assert_eq(ret, 0);
+			wait_for_pageflip(data->gfx_fd);
+		}
+
+		valid_tests++;
+		cleanup_crtc(data, output, plane);
 	}
 	igt_require_f(valid_tests, "no valid crtc/connector combinations found\n");
 }
 
-static void test_plane_rotation_ytiled_obj(data_t *data, enum igt_plane plane_type)
+static void test_plane_rotation_ytiled_obj(data_t *data,
+					   igt_output_t *output,
+					   int plane_type)
 {
 	igt_display_t *display = &data->display;
 	uint64_t tiling = LOCAL_I915_FORMAT_MOD_Y_TILED;
@@ -329,22 +329,20 @@ static void test_plane_rotation_ytiled_obj(data_t *data, enum igt_plane plane_ty
 	int bpp = igt_drm_format_to_bpp(format);
 	enum igt_commit_style commit = COMMIT_LEGACY;
 	int fd = data->gfx_fd;
-	igt_output_t *output = &display->outputs[0];
 	igt_plane_t *plane;
 	drmModeModeInfo *mode;
 	unsigned int stride, size, w, h;
 	uint32_t gem_handle;
 	int ret;
 
-	igt_require(output != NULL && output->valid == true);
-
-	plane = igt_output_get_plane(output, plane_type);
+	plane = igt_output_get_plane_type(output, plane_type);
 	igt_require(igt_plane_supports_rotation(plane));
 
-	if (plane_type == IGT_PLANE_PRIMARY || plane_type == IGT_PLANE_CURSOR) {
-		igt_require(data->display.has_universal_planes);
+	if (plane_type == DRM_PLANE_TYPE_PRIMARY || plane_type == DRM_PLANE_TYPE_CURSOR)
 		commit = COMMIT_UNIVERSAL;
-	}
+
+	if (plane_type == DRM_PLANE_TYPE_CURSOR)
+		igt_require(display->has_cursor_plane);
 
 	if (data->display.is_atomic)
 		commit = COMMIT_ATOMIC;
@@ -360,7 +358,7 @@ static void test_plane_rotation_ytiled_obj(data_t *data, enum igt_plane plane_ty
 
 	gem_handle = gem_create(fd, size);
 	ret = __gem_set_tiling(fd, gem_handle, I915_TILING_Y, stride);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	do_or_die(__kms_addfb(fd, gem_handle, w, h, stride,
 		  format, tiling, LOCAL_DRM_MODE_FB_MODIFIERS,
@@ -374,19 +372,26 @@ static void test_plane_rotation_ytiled_obj(data_t *data, enum igt_plane plane_ty
 
 	igt_plane_set_rotation(plane, data->rotation);
 	igt_plane_set_fb(plane, &data->fb);
+	igt_plane_set_size(plane, h, w);
+
+	if (commit < COMMIT_ATOMIC)
+		drmModeObjectSetProperty(fd, plane->drm_plane->plane_id,
+					DRM_MODE_OBJECT_PLANE,
+					plane->rotation_property,
+					plane->rotation);
 
-	drmModeObjectSetProperty(fd, plane->drm_plane->plane_id,
-				 DRM_MODE_OBJECT_PLANE,
-				 plane->rotation_property,
-				 plane->rotation);
 	ret = igt_display_try_commit2(display, commit);
 
+	igt_output_set_pipe(output, PIPE_NONE);
+
 	kmstest_restore_vt_mode();
 	igt_remove_fb(fd, &data->fb);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 }
 
-static void test_plane_rotation_exhaust_fences(data_t *data, enum igt_plane plane_type)
+static void test_plane_rotation_exhaust_fences(data_t *data,
+					       igt_output_t *output,
+					       int plane_type)
 {
 	igt_display_t *display = &data->display;
 	uint64_t tiling = LOCAL_I915_FORMAT_MOD_Y_TILED;
@@ -394,7 +399,6 @@ static void test_plane_rotation_exhaust_fences(data_t *data, enum igt_plane plan
 	int bpp = igt_drm_format_to_bpp(format);
 	enum igt_commit_style commit = COMMIT_LEGACY;
 	int fd = data->gfx_fd;
-	igt_output_t *output = &display->outputs[0];
 	igt_plane_t *plane;
 	drmModeModeInfo *mode;
 	data_t data2[MAX_FENCES+1] = {};
@@ -403,15 +407,14 @@ static void test_plane_rotation_exhaust_fences(data_t *data, enum igt_plane plan
 	uint64_t total_aperture_size, total_fbs_size;
 	int i, ret;
 
-	igt_require(output != NULL && output->valid == true);
-
-	plane = igt_output_get_plane(output, plane_type);
+	plane = igt_output_get_plane_type(output, plane_type);
 	igt_require(igt_plane_supports_rotation(plane));
 
-	if (plane_type == IGT_PLANE_PRIMARY || plane_type == IGT_PLANE_CURSOR) {
-		igt_require(data->display.has_universal_planes);
+	if (plane_type == DRM_PLANE_TYPE_PRIMARY || plane_type == DRM_PLANE_TYPE_CURSOR)
 		commit = COMMIT_UNIVERSAL;
-	}
+
+	if (plane_type == DRM_PLANE_TYPE_CURSOR)
+		igt_require(display->has_cursor_plane);
 
 	if (data->display.is_atomic)
 		commit = COMMIT_ATOMIC;
@@ -466,14 +469,15 @@ static void test_plane_rotation_exhaust_fences(data_t *data, enum igt_plane plan
 		}
 
 		igt_plane_set_rotation(plane, IGT_ROTATION_90);
+		igt_plane_set_size(plane, h, w);
 
 		drmModeObjectSetProperty(fd, plane->drm_plane->plane_id,
 					 DRM_MODE_OBJECT_PLANE,
 					 plane->rotation_property,
 					 plane->rotation);
-		ret = igt_display_try_commit2(display, commit);
+		igt_display_commit2(display, commit);
 		if (ret) {
-			igt_warn("failed to commit hardware rotated fb\n");
+			igt_warn("failed to commit hardware rotated fb: %i\n", ret);
 			goto err_commit;
 		}
 	}
@@ -484,11 +488,20 @@ err_alloc:
 
 	i--;
 err_commit:
+	igt_plane_set_fb(plane, NULL);
+	igt_plane_set_rotation(plane, IGT_ROTATION_0);
+
+	if (commit < COMMIT_ATOMIC)
+		igt_display_commit2(display, commit);
+
+	igt_output_set_pipe(output, PIPE_NONE);
+	igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
+
 	for (; i >= 0; i--)
 		igt_remove_fb(fd, &data2[i].fb);
 
 	kmstest_restore_vt_mode();
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 }
 
 igt_main
@@ -510,41 +523,41 @@ igt_main
 	}
 	igt_subtest_f("primary-rotation-180") {
 		data.rotation = IGT_ROTATION_180;
-		test_plane_rotation(&data, IGT_PLANE_PRIMARY);
+		test_plane_rotation(&data, DRM_PLANE_TYPE_PRIMARY);
 	}
 
 	igt_subtest_f("sprite-rotation-180") {
 		data.rotation = IGT_ROTATION_180;
-		test_plane_rotation(&data, IGT_PLANE_2);
+		test_plane_rotation(&data, DRM_PLANE_TYPE_OVERLAY);
 	}
 
 	igt_subtest_f("cursor-rotation-180") {
 		data.rotation = IGT_ROTATION_180;
-		test_plane_rotation(&data, IGT_PLANE_CURSOR);
+		test_plane_rotation(&data, DRM_PLANE_TYPE_CURSOR);
 	}
 
 	igt_subtest_f("primary-rotation-90") {
 		igt_require(gen >= 9);
 		data.rotation = IGT_ROTATION_90;
-		test_plane_rotation(&data, IGT_PLANE_PRIMARY);
+		test_plane_rotation(&data, DRM_PLANE_TYPE_PRIMARY);
 	}
 
 	igt_subtest_f("primary-rotation-270") {
 		igt_require(gen >= 9);
 		data.rotation = IGT_ROTATION_270;
-		test_plane_rotation(&data, IGT_PLANE_PRIMARY);
+		test_plane_rotation(&data, DRM_PLANE_TYPE_PRIMARY);
 	}
 
 	igt_subtest_f("sprite-rotation-90") {
 		igt_require(gen >= 9);
 		data.rotation = IGT_ROTATION_90;
-		test_plane_rotation(&data, IGT_PLANE_2);
+		test_plane_rotation(&data, DRM_PLANE_TYPE_OVERLAY);
 	}
 
 	igt_subtest_f("sprite-rotation-270") {
 		igt_require(gen >= 9);
 		data.rotation = IGT_ROTATION_270;
-		test_plane_rotation(&data, IGT_PLANE_2);
+		test_plane_rotation(&data, DRM_PLANE_TYPE_OVERLAY);
 	}
 
 	igt_subtest_f("sprite-rotation-90-pos-100-0") {
@@ -552,7 +565,7 @@ igt_main
 		data.rotation = IGT_ROTATION_90;
 		data.pos_x = 100,
 		data.pos_y = 0;
-		test_plane_rotation(&data, IGT_PLANE_2);
+		test_plane_rotation(&data, DRM_PLANE_TYPE_OVERLAY);
 	}
 
 	igt_subtest_f("bad-pixel-format") {
@@ -561,7 +574,7 @@ igt_main
 		data.pos_y = 0;
 		data.rotation = IGT_ROTATION_90;
 		data.override_fmt = DRM_FORMAT_RGB565;
-		test_plane_rotation(&data, IGT_PLANE_PRIMARY);
+		test_plane_rotation(&data, DRM_PLANE_TYPE_PRIMARY);
 	}
 
 	igt_subtest_f("bad-tiling") {
@@ -569,7 +582,7 @@ igt_main
 		data.override_fmt = 0;
 		data.rotation = IGT_ROTATION_90;
 		data.override_tiling = LOCAL_DRM_FORMAT_MOD_NONE;
-		test_plane_rotation(&data, IGT_PLANE_PRIMARY);
+		test_plane_rotation(&data, DRM_PLANE_TYPE_PRIMARY);
 	}
 
 	igt_subtest_f("primary-rotation-90-flip-stress") {
@@ -577,18 +590,46 @@ igt_main
 		data.override_tiling = 0;
 		data.flip_stress = 60;
 		data.rotation = IGT_ROTATION_90;
-		test_plane_rotation(&data, IGT_PLANE_PRIMARY);
+		test_plane_rotation(&data, DRM_PLANE_TYPE_PRIMARY);
 	}
 
 	igt_subtest_f("primary-rotation-90-Y-tiled") {
+		enum pipe pipe;
+		igt_output_t *output;
+		int valid_tests = 0;
+
 		igt_require(gen >= 9);
 		data.rotation = IGT_ROTATION_90;
-		test_plane_rotation_ytiled_obj(&data, IGT_PLANE_PRIMARY);
+
+		for_each_pipe_with_valid_output(&data.display, pipe, output) {
+			igt_output_set_pipe(output, pipe);
+
+			test_plane_rotation_ytiled_obj(&data, output, DRM_PLANE_TYPE_PRIMARY);
+
+			valid_tests++;
+			break;
+		}
+
+		igt_require_f(valid_tests, "no valid crtc/connector combinations found\n");
 	}
 
 	igt_subtest_f("exhaust-fences") {
+		enum pipe pipe;
+		igt_output_t *output;
+		int valid_tests = 0;
+
 		igt_require(gen >= 9);
-		test_plane_rotation_exhaust_fences(&data, IGT_PLANE_PRIMARY);
+
+		for_each_pipe_with_valid_output(&data.display, pipe, output) {
+			igt_output_set_pipe(output, pipe);
+
+			test_plane_rotation_exhaust_fences(&data, output, DRM_PLANE_TYPE_PRIMARY);
+
+			valid_tests++;
+			break;
+		}
+
+		igt_require_f(valid_tests, "no valid crtc/connector combinations found\n");
 	}
 
 	igt_fixture {
